# Generated by Django 2.2.6 on 2019-10-26 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('silk', '0006_fix_request_prof_file_blank'),
    ]

    operations = [
        migrations.AddField(
            model_name='sqlquery',
            name='identifier',
            field=models.IntegerField(default=-1),
        ),
    ]
