.very-good-font-color {
  color: #bac54b; }

.good-font-color {
  color: #c3a948; }

.ok-font-color {
  color: #c08245; }

.bad-font-color {
  color: #be5b43; }

.very-bad-font-color {
  color: #b9424f; }
