.cell {
  display: inline-block;
  background-color: transparent;
  padding: 10px;
  margin-left: 10px;
  margin-top: 10px;
  border-radius: 4px;
  transition: background-color 0.15s ease, color 0.15s ease;
  div {
    margin: 2px;
  }
  .timestamp-div {
    margin-bottom: 15px;
    font-size: 13px;
  }
  .meta {
    font-size: 12px;
    color: #be5b43;
    .unit {
      font-size: 9px;
      font-weight: lighter !important;
    }

  }
  .method-div {
    font-weight: bold;
    font-size: 20px;
  }
  .path-div {
    font-size: 18px;
    margin-bottom: 15px;
  }
}
