# Generated by Django 1.9.7 on 2016-07-08 13:19

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Blind',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('photo', models.ImageField(upload_to=b'products')),
                ('name', models.TextField()),
                ('child_safe', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
